@@ -4,10 +4,11 @@ import (
 	"context"
 	"fmt"
 	"reflect"
+	"strings"
 	"time"
 
 	"github.com/clarketm/json"
-	igntypes "github.com/coreos/ignition/config/v2_2/types"
+	ign3types "github.com/coreos/ignition/v2/config/v3_2/types"
 	"github.com/golang/glog"
 	apicfgv1 "github.com/openshift/api/config/v1"
 	apioperatorsv1alpha1 "github.com/openshift/api/operator/v1alpha1"
@@ -68,11 +69,10 @@ var updateBackoff = wait.Backoff{
 
 // Controller defines the container runtime config controller.
 type Controller struct {
-	namespace, templatesDir string
+	templatesDir string
 
 	client        mcfgclientset.Interface
 	configClient  configclientset.Interface
-	kubeClient    clientset.Interface
 	eventRecorder record.EventRecorder
 
 	syncHandler                   func(mcp string) error
@@ -99,15 +99,11 @@ type Controller struct {
 
 	queue    workqueue.RateLimitingInterface
 	imgQueue workqueue.RateLimitingInterface
-
-	// we need this method to mock out patch calls in unit until https://github.com/openshift/machine-config-operator/pull/611#issuecomment-481397185
-	// which is probably going to be in kube 1.14
-	patchContainerRuntimeConfigsFunc func(string, []byte) error
 }
 
 // New returns a new container runtime config controller
 func New(
-	namespace, templatesDir string,
+	templatesDir string,
 	mcpInformer mcfginformersv1.MachineConfigPoolInformer,
 	ccInformer mcfginformersv1.ControllerConfigInformer,
 	mcrInformer mcfginformersv1.ContainerRuntimeConfigInformer,
@@ -123,11 +119,9 @@ func New(
 	eventBroadcaster.StartRecordingToSink(&coreclientsetv1.EventSinkImpl{Interface: kubeClient.CoreV1().Events("")})
 
 	ctrl := &Controller{
-		namespace:     namespace,
 		templatesDir:  templatesDir,
 		client:        mcfgClient,
 		configClient:  configClient,
-		kubeClient:    kubeClient,
 		eventRecorder: eventBroadcaster.NewRecorder(scheme.Scheme, corev1.EventSource{Component: "machineconfigcontroller-containerruntimeconfigcontroller"}),
 		queue:         workqueue.NewNamedRateLimitingQueue(workqueue.DefaultControllerRateLimiter(), "machineconfigcontroller-containerruntimeconfigcontroller"),
 		imgQueue:      workqueue.NewRateLimitingQueue(workqueue.DefaultControllerRateLimiter()),
@@ -173,12 +167,6 @@ func New(
 	ctrl.clusterVersionLister = clusterVersionInformer.Lister()
 	ctrl.clusterVersionListerSynced = clusterVersionInformer.Informer().HasSynced
 
-	ctrl.patchContainerRuntimeConfigsFunc = ctrl.patchContainerRuntimeConfigs
-	// Add to the queue to trigger a sync when an upgrade happens
-	// this ensures that the capabilities MC is created on an upgrade
-	// This will be removed in the next version
-	ctrl.queue.Add("force-sync-on-upgrade")
-
 	return ctrl
 }
 
@@ -385,7 +373,7 @@ func (ctrl *Controller) handleImgErr(err error, key interface{}) {
 }
 
 // generateOriginalContainerRuntimeConfigs returns rendered default storage, registries and policy config files
-func generateOriginalContainerRuntimeConfigs(templateDir string, cc *mcfgv1.ControllerConfig, role string) (*igntypes.File, *igntypes.File, *igntypes.File, error) {
+func generateOriginalContainerRuntimeConfigs(templateDir string, cc *mcfgv1.ControllerConfig, role string) (*ign3types.File, *ign3types.File, *ign3types.File, error) {
 	// Render the default templates
 	rc := &mtmpl.RenderConfig{ControllerConfigSpec: &cc.Spec}
 	generatedConfigs, err := mtmpl.GenerateMachineConfigsForRole(rc, role, templateDir)
@@ -394,7 +382,7 @@ func generateOriginalContainerRuntimeConfigs(templateDir string, cc *mcfgv1.Cont
 	}
 	// Find generated storage.conf, registries.conf, and policy.json
 	var (
-		config, gmcStorageConfig, gmcRegistriesConfig, gmcPolicyJSON *igntypes.File
+		config, gmcStorageConfig, gmcRegistriesConfig, gmcPolicyJSON *ign3types.File
 		errStorage, errRegistries, errPolicy                         error
 	)
 	// Find storage config
@@ -459,8 +447,28 @@ func (ctrl *Controller) syncStatusOnly(cfg *mcfgv1.ContainerRuntimeConfig, err e
 	return err
 }
 
+// addAnnotation adds the annotions for a ctrcfg object with the given annotationKey and annotationVal
+func (ctrl *Controller) addAnnotation(cfg *mcfgv1.ContainerRuntimeConfig, annotationKey, annotationVal string) error {
+	annotationUpdateErr := retry.RetryOnConflict(updateBackoff, func() error {
+		newcfg, getErr := ctrl.mccrLister.Get(cfg.Name)
+		if getErr != nil {
+			return getErr
+		}
+		newcfg.SetAnnotations(map[string]string{
+			annotationKey: annotationVal,
+		})
+		_, updateErr := ctrl.client.MachineconfigurationV1().ContainerRuntimeConfigs().Update(context.TODO(), newcfg, metav1.UpdateOptions{})
+		return updateErr
+	})
+	if annotationUpdateErr != nil {
+		glog.Warningf("error updating the container runtime config with annotation key %q and value %q: %v", annotationKey, annotationVal, annotationUpdateErr)
+	}
+	return annotationUpdateErr
+}
+
 // syncContainerRuntimeConfig will sync the ContainerRuntimeconfig with the given key.
 // This function is not meant to be invoked concurrently with the same key.
+// nolint: gocyclo
 func (ctrl *Controller) syncContainerRuntimeConfig(key string) error {
 	startTime := time.Now()
 	glog.V(4).Infof("Started syncing ContainerRuntimeconfig %q (%v)", key, startTime)
@@ -468,17 +476,6 @@ func (ctrl *Controller) syncContainerRuntimeConfig(key string) error {
 		glog.V(4).Infof("Finished syncing ContainerRuntimeconfig %q (%v)", key, time.Since(startTime))
 	}()
 
-	// First let's create the MC for the default capabilities file
-	// This will be removed in the next version
-	if err := ctrl.createCapabilitiesMC(); err != nil {
-		return fmt.Errorf("failed to create the crio capabilitie MC: %v", err)
-	}
-	// If the key is set to force-sync-on-upgrade, then we can return after creating
-	// the capabilities MC.
-	if key == "force-sync-on-upgrade" {
-		return nil
-	}
-
 	_, name, err := cache.SplitMetaNamespaceKey(key)
 	if err != nil {
 		return err
@@ -522,10 +519,19 @@ func (ctrl *Controller) syncContainerRuntimeConfig(key string) error {
 		return ctrl.syncStatusOnly(cfg, err)
 	}
 
+	if len(mcpPools) == 0 {
+		err := fmt.Errorf("containerRuntimeConfig %v does not match any MachineConfigPools", key)
+		glog.V(2).Infof("%v", err)
+		return ctrl.syncStatusOnly(cfg, err)
+	}
+
 	for _, pool := range mcpPools {
 		role := pool.Name
 		// Get MachineConfig
-		managedKey := getManagedKeyCtrCfg(pool)
+		managedKey, err := getManagedKeyCtrCfg(pool, ctrl.client, cfg)
+		if err != nil {
+			return ctrl.syncStatusOnly(cfg, err, "could not get ctrcfg key: %v", err)
+		}
 		mc, err := ctrl.client.MachineconfigurationV1().MachineConfigs().Get(context.TODO(), managedKey, metav1.GetOptions{})
 		isNotFound := errors.IsNotFound(err)
 		if err != nil && !isNotFound {
@@ -534,8 +540,9 @@ func (ctrl *Controller) syncContainerRuntimeConfig(key string) error {
 		// If we have seen this generation and the sync didn't fail, then skip
 		if !isNotFound && cfg.Status.ObservedGeneration >= cfg.Generation && cfg.Status.Conditions[len(cfg.Status.Conditions)-1].Type == mcfgv1.ContainerRuntimeConfigSuccess {
 			// But we still need to compare the generated controller version because during an upgrade we need a new one
-			if mc.Annotations[ctrlcommon.GeneratedByControllerVersionAnnotationKey] == version.Hash {
-				continue
+			mcCtrlVersion := mc.Annotations[ctrlcommon.GeneratedByControllerVersionAnnotationKey]
+			if mcCtrlVersion == version.Hash {
+				return nil
 			}
 		}
 		// Generate the original ContainerRuntimeConfig
@@ -563,10 +570,19 @@ func (ctrl *Controller) syncContainerRuntimeConfig(key string) error {
 
 		if isNotFound {
 			tempIgnCfg := ctrlcommon.NewIgnConfig()
-			mc, err = mtmpl.MachineConfigFromIgnConfig(role, managedKey, tempIgnCfg)
+			mc, err = ctrlcommon.MachineConfigFromIgnConfig(role, managedKey, tempIgnCfg)
 			if err != nil {
 				return ctrl.syncStatusOnly(cfg, err, "could not create MachineConfig from new Ignition config: %v", err)
 			}
+			_, ok := cfg.GetAnnotations()[ctrlcommon.MCNameSuffixAnnotationKey]
+			arr := strings.Split(managedKey, "-")
+			// If the MC name suffix annotation does not exist and the managed key value returned has a suffix, then add the MC name
+			// suffix annotation and suffix value to the ctrcfg object
+			if len(arr) > 4 && !ok {
+				if err := ctrl.addAnnotation(cfg, ctrlcommon.MCNameSuffixAnnotationKey, arr[len(arr)-1]); err != nil {
+					return ctrl.syncStatusOnly(cfg, err, "could not update annotation for containerRuntimeConfig")
+				}
+			}
 		}
 
 		ctrRuntimeConfigIgn := createNewIgnition(configFileList)
@@ -606,8 +622,11 @@ func (ctrl *Controller) syncContainerRuntimeConfig(key string) error {
 
 // mergeConfigChanges retrieves the original/default config data from the templates, decodes it and merges in the changes given by the Custom Resource.
 // It then encodes the new data and returns it.
-func (ctrl *Controller) mergeConfigChanges(origFile *igntypes.File, cfg *mcfgv1.ContainerRuntimeConfig, update updateConfigFunc) ([]byte, error) {
-	dataURL, err := dataurl.DecodeString(origFile.Contents.Source)
+func (ctrl *Controller) mergeConfigChanges(origFile *ign3types.File, cfg *mcfgv1.ContainerRuntimeConfig, update updateConfigFunc) ([]byte, error) {
+	if origFile.Contents.Source == nil {
+		return nil, ctrl.syncStatusOnly(cfg, fmt.Errorf("original Container Runtime config is empty"))
+	}
+	dataURL, err := dataurl.DecodeString(*origFile.Contents.Source)
 	if err != nil {
 		return nil, ctrl.syncStatusOnly(cfg, err, "could not decode original Container Runtime config: %v", err)
 	}
@@ -618,95 +637,6 @@ func (ctrl *Controller) mergeConfigChanges(origFile *igntypes.File, cfg *mcfgv1.
 	return cfgTOML, ctrl.syncStatusOnly(cfg, nil)
 }
 
-// createCapabilitiesMC creates a Machine Config for the crio drop-in capabilities file.
-// We are dropping the NET_RAW and SYS_CHROOT capabilitie by default and in order to not break
-// existing clusters, we are creating a drop-in file at /etc/crio/crio.conf.d that has NET_RAW
-// and SYS_CHROOT as part of the default capabilities list. Users then have the option to delete the
-// capabilities MC that creates this drop-in file.
-// Create a crio-capabilities config map to keep track of whether this capabilities MC was already created
-// once. This ensures that if the user deletes the capabilities MC, the controller doesn't recreate it on a
-// resync.
-func (ctrl *Controller) createCapabilitiesMC() error {
-	var configMapName = "crio-capabilities"
-
-	// Check if the crio-capabilities config map exists in the openshift-machine-config-operator namespace
-	capCM, err := ctrl.kubeClient.CoreV1().ConfigMaps(ctrl.namespace).Get(context.TODO(), configMapName, metav1.GetOptions{})
-	capCMIsNotFound := errors.IsNotFound(err)
-	if err != nil && !capCMIsNotFound {
-		return fmt.Errorf("error checking for %s config map: %v", configMapName, err)
-	}
-	// If the crio-capabilities config map exists, that means the capabilities MC was already created
-	// so we should not create this MC again and return
-	if capCM != nil && !capCMIsNotFound {
-		return nil
-	}
-
-	sel, err := metav1.LabelSelectorAsSelector(metav1.AddLabelToSelector(&metav1.LabelSelector{}, builtInLabelKey, ""))
-	if err != nil {
-		return err
-	}
-	// Find all the MachineConfigPools
-	mcpPoolsAll, err := ctrl.mcpLister.List(sel)
-	if err != nil {
-		return err
-	}
-
-	// Create the capabilities MC for all the available pools
-	for _, pool := range mcpPoolsAll {
-		managedKey := getManagedKeyCaps(pool)
-		mc, err := ctrl.client.MachineconfigurationV1().MachineConfigs().Get(context.TODO(), managedKey, metav1.GetOptions{})
-		isNotFound := errors.IsNotFound(err)
-		if err != nil && !isNotFound {
-			return fmt.Errorf("error checking for %s machine config: %v", managedKey, err)
-		}
-		// continue to the next MC if this already exists
-		if mc != nil && !isNotFound {
-			continue
-		}
-
-		tempIgnCfg := ctrlcommon.NewIgnConfig()
-		mc, err = mtmpl.MachineConfigFromIgnConfig(pool.Name, managedKey, tempIgnCfg)
-		if err != nil {
-			return fmt.Errorf("could not create capabilities MachineConfig from new Ignition config: %v", err)
-		}
-		rawCapsIgnition, err := json.Marshal(createNewIgnition(createDefaultCapsFile()))
-		if err != nil {
-			return fmt.Errorf("error marshalling capabilities config ignition: %v", err)
-		}
-		mc.Spec.Config.Raw = rawCapsIgnition
-		// Create the capabilities MC
-		if err := retry.RetryOnConflict(updateBackoff, func() error {
-			_, err = ctrl.client.MachineconfigurationV1().MachineConfigs().Create(context.TODO(), mc, metav1.CreateOptions{})
-			return err
-		}); err != nil {
-			return fmt.Errorf("could not create MachineConfig for capabilities: %v", err)
-		}
-		glog.Infof("Applied Capabilities MC %v on MachineConfigPool %v", managedKey, pool.Name)
-	}
-
-	// Create the config map for capabilities so we know that the capabilities MC has been created
-	capCM.Name = configMapName
-	capCM.Namespace = ctrl.namespace
-	if _, err := ctrl.kubeClient.CoreV1().ConfigMaps(ctrl.namespace).Create(context.TODO(), capCM, metav1.CreateOptions{}); err != nil {
-		return fmt.Errorf("error creating %s config map: %v", configMapName, err)
-	}
-	return nil
-}
-
-// RunCapsBootstrap creates the capabilities mc on bootstrap
-func RunCapsBootstrap(mcpPools []*mcfgv1.MachineConfigPool) ([]*mcfgv1.MachineConfig, error) {
-	var res []*mcfgv1.MachineConfig
-	for _, pool := range mcpPools {
-		capsIgn := createNewIgnition(createDefaultCapsFile())
-		mc, err := mtmpl.MachineConfigFromIgnConfig(pool.Name, getManagedKeyCaps(pool), capsIgn)
-		if err != nil {
-			return nil, err
-		}
-		res = append(res, mc)
-	}
-	return res, nil
-}
-
 func (ctrl *Controller) syncImageConfig(key string) error {
 	startTime := time.Now()
 	glog.V(4).Infof("Started syncing ImageConfig %q (%v)", key, startTime)
@@ -737,12 +667,15 @@ func (ctrl *Controller) syncImageConfig(key string) error {
 		return err
 	}
 
-	// Go through the registries in the image spec to get and validate the blocked registries
-	blockedRegs, err := getValidBlockedRegistries(&clusterVersionCfg.Status, &imgcfg.Spec)
-	if err != nil && err != errParsingReference {
-		glog.V(2).Infof("%v, skipping....", err)
-	} else if err == errParsingReference {
-		return err
+	var blockedRegs []string
+	if clusterVersionCfg != nil {
+		// Go through the registries in the image spec to get and validate the blocked registries
+		blockedRegs, err = getValidBlockedRegistries(clusterVersionCfg.Status.Desired.Image, &imgcfg.Spec)
+		if err != nil && err != errParsingReference {
+			glog.V(2).Infof("%v, skipping....", err)
+		} else if err == errParsingReference {
+			return err
+		}
 	}
 
 	// Get ControllerConfig
@@ -773,10 +706,14 @@ func (ctrl *Controller) syncImageConfig(key string) error {
 		applied := true
 		role := pool.Name
 		// Get MachineConfig
-		managedKey := getManagedKeyReg(pool)
+		managedKey, err := getManagedKeyReg(pool, ctrl.client)
+		if err != nil {
+			return err
+		}
 		if err := retry.RetryOnConflict(updateBackoff, func() error {
 			registriesIgn, err := registriesConfigIgnition(ctrl.templatesDir, controllerConfig, role,
-				imgcfg.Spec.RegistrySources.InsecureRegistries, blockedRegs, imgcfg.Spec.RegistrySources.AllowedRegistries, icspRules)
+				imgcfg.Spec.RegistrySources.InsecureRegistries, blockedRegs, imgcfg.Spec.RegistrySources.AllowedRegistries,
+				imgcfg.Spec.RegistrySources.ContainerRuntimeSearchRegistries, icspRules)
 			if err != nil {
 				return err
 			}
@@ -800,7 +737,7 @@ func (ctrl *Controller) syncImageConfig(key string) error {
 			}
 			if isNotFound {
 				tempIgnCfg := ctrlcommon.NewIgnConfig()
-				mc, err = mtmpl.MachineConfigFromIgnConfig(role, managedKey, tempIgnCfg)
+				mc, err = ctrlcommon.MachineConfigFromIgnConfig(role, managedKey, tempIgnCfg)
 				if err != nil {
 					return fmt.Errorf("could not create MachineConfig from new Ignition config: %v", err)
 				}
@@ -837,7 +774,7 @@ func (ctrl *Controller) syncImageConfig(key string) error {
 }
 
 func registriesConfigIgnition(templateDir string, controllerConfig *mcfgv1.ControllerConfig, role string,
-	insecureRegs, blockedRegs, allowedRegs []string, icspRules []*apioperatorsv1alpha1.ImageContentSourcePolicy) (*igntypes.Config, error) {
+	insecureRegs, blockedRegs, allowedRegs, searchRegs []string, icspRules []*apioperatorsv1alpha1.ImageContentSourcePolicy) (*ign3types.Config, error) {
 
 	var (
 		registriesTOML []byte
@@ -851,7 +788,10 @@ func registriesConfigIgnition(templateDir string, controllerConfig *mcfgv1.Contr
 	}
 
 	if insecureRegs != nil || blockedRegs != nil || len(icspRules) != 0 {
-		dataURL, err := dataurl.DecodeString(originalRegistriesIgn.Contents.Source)
+		if originalRegistriesIgn.Contents.Source == nil {
+			return nil, fmt.Errorf("original registries config is empty")
+		}
+		dataURL, err := dataurl.DecodeString(*originalRegistriesIgn.Contents.Source)
 		if err != nil {
 			return nil, fmt.Errorf("could not decode original registries config: %v", err)
 		}
@@ -861,7 +801,10 @@ func registriesConfigIgnition(templateDir string, controllerConfig *mcfgv1.Contr
 		}
 	}
 	if blockedRegs != nil || allowedRegs != nil {
-		dataURL, err := dataurl.DecodeString(originalPolicyIgn.Contents.Source)
+		if originalPolicyIgn.Contents.Source == nil {
+			return nil, fmt.Errorf("original policy json is empty")
+		}
+		dataURL, err := dataurl.DecodeString(*originalPolicyIgn.Contents.Source)
 		if err != nil {
 			return nil, fmt.Errorf("could not decode original policy json: %v", err)
 		}
@@ -870,32 +813,55 @@ func registriesConfigIgnition(templateDir string, controllerConfig *mcfgv1.Contr
 			return nil, fmt.Errorf("could not update policy json with new changes: %v", err)
 		}
 	}
-	registriesIgn := createNewIgnition([]generatedConfigFile{
+	generatedConfigFileList := []generatedConfigFile{
 		{filePath: registriesConfigPath, data: registriesTOML},
 		{filePath: policyConfigPath, data: policyJSON},
-	})
+	}
+	if searchRegs != nil {
+		generatedConfigFileList = append(generatedConfigFileList, updateSearchRegistriesConfig(searchRegs)...)
+	}
+
+	registriesIgn := createNewIgnition(generatedConfigFileList)
 	return &registriesIgn, nil
 }
 
 // RunImageBootstrap generates MachineConfig objects for mcpPools that would have been generated by syncImageConfig,
 // except that mcfgv1.Image is not available.
-func RunImageBootstrap(templateDir string, controllerConfig *mcfgv1.ControllerConfig, mcpPools []*mcfgv1.MachineConfigPool, icspRules []*apioperatorsv1alpha1.ImageContentSourcePolicy) ([]*mcfgv1.MachineConfig, error) {
-	// ImageConfig is not available.
-	insecureRegs := []string(nil)
-	blockedRegs := []string(nil)
-	allowedRegs := []string(nil)
+func RunImageBootstrap(templateDir string, controllerConfig *mcfgv1.ControllerConfig, mcpPools []*mcfgv1.MachineConfigPool, icspRules []*apioperatorsv1alpha1.ImageContentSourcePolicy, imgCfg *apicfgv1.Image) ([]*mcfgv1.MachineConfig, error) {
+	var (
+		insecureRegs []string
+		blockedRegs  []string
+		allowedRegs  []string
+		searchRegs   []string
+		err          error
+	)
+
+	// Read the search, insecure, blocked, and allowed registries from the cluster-wide Image CR if it is not nil
+	if imgCfg != nil {
+		insecureRegs = imgCfg.Spec.RegistrySources.InsecureRegistries
+		allowedRegs = imgCfg.Spec.RegistrySources.AllowedRegistries
+		searchRegs = imgCfg.Spec.RegistrySources.ContainerRuntimeSearchRegistries
+		blockedRegs, err = getValidBlockedRegistries(controllerConfig.Spec.ReleaseImage, &imgCfg.Spec)
+		if err != nil && err != errParsingReference {
+			glog.V(2).Infof("%v, skipping....", err)
+		} else if err == errParsingReference {
+			return nil, err
+		}
+	}
 
 	var res []*mcfgv1.MachineConfig
 	for _, pool := range mcpPools {
 		role := pool.Name
-		managedKey := getManagedKeyReg(pool)
-
-		registriesIgn, err := registriesConfigIgnition(templateDir, controllerConfig, role,
-			insecureRegs, blockedRegs, allowedRegs, icspRules)
+		managedKey, err := getManagedKeyReg(pool, nil)
 		if err != nil {
 			return nil, err
 		}
-		mc, err := mtmpl.MachineConfigFromIgnConfig(role, managedKey, registriesIgn)
+		registriesIgn, err := registriesConfigIgnition(templateDir, controllerConfig, role,
+			insecureRegs, blockedRegs, allowedRegs, searchRegs, icspRules)
+		if err != nil {
+			return nil, err
+		}
+		mc, err := ctrlcommon.MachineConfigFromIgnConfig(role, managedKey, registriesIgn)
 		if err != nil {
 			return nil, err
 		}
@@ -940,7 +906,7 @@ func (ctrl *Controller) popFinalizerFromContainerRuntimeConfig(ctrCfg *mcfgv1.Co
 		if err != nil {
 			return err
 		}
-		return ctrl.patchContainerRuntimeConfigsFunc(ctrCfg.Name, patch)
+		return ctrl.patchContainerRuntimeConfigs(ctrCfg.Name, patch)
 	})
 }
 
@@ -982,7 +948,7 @@ func (ctrl *Controller) addFinalizerToContainerRuntimeConfig(ctrCfg *mcfgv1.Cont
 		if err != nil {
 			return err
 		}
-		return ctrl.patchContainerRuntimeConfigsFunc(ctrCfg.Name, patch)
+		return ctrl.patchContainerRuntimeConfigs(ctrCfg.Name, patch)
 	})
 }
 
