@@ -14,7 +14,6 @@ import (
 	"github.com/stretchr/testify/require"
 	"github.com/vincent-petithory/dataurl"
 
-	k8sv1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/equality"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
@@ -27,8 +26,7 @@ import (
 	"k8s.io/client-go/tools/cache"
 	"k8s.io/client-go/tools/record"
 
-	ign "github.com/coreos/ignition/config/v2_2"
-	igntypes "github.com/coreos/ignition/config/v2_2/types"
+	ign3types "github.com/coreos/ignition/v2/config/v3_2/types"
 	apicfgv1 "github.com/openshift/api/config/v1"
 	apioperatorsv1alpha1 "github.com/openshift/api/operator/v1alpha1"
 	fakeconfigv1client "github.com/openshift/client-go/config/clientset/versioned/fake"
@@ -61,7 +59,6 @@ type fixture struct {
 	client         *fake.Clientset
 	imgClient      *fakeconfigv1client.Clientset
 	operatorClient *fakeoperatorclient.Clientset
-	kubeClient     *k8sfake.Clientset
 
 	ccLister   []*mcfgv1.ControllerConfig
 	mcpLister  []*mcfgv1.MachineConfigPool
@@ -75,7 +72,6 @@ type fixture struct {
 	objects         []runtime.Object
 	imgObjects      []runtime.Object
 	operatorObjects []runtime.Object
-	k8sObjects      []runtime.Object
 }
 
 func newFixture(t *testing.T) *fixture {
@@ -104,13 +100,22 @@ func (f *fixture) validateActions() {
 	}
 }
 
-func newControllerConfig(name, platform string) *mcfgv1.ControllerConfig {
+func newControllerConfig(name string, platform apicfgv1.PlatformType) *mcfgv1.ControllerConfig {
 	cc := &mcfgv1.ControllerConfig{
 		TypeMeta:   metav1.TypeMeta{APIVersion: mcfgv1.SchemeGroupVersion.String()},
 		ObjectMeta: metav1.ObjectMeta{Name: name, UID: types.UID(utilrand.String(5))},
 		Spec: mcfgv1.ControllerConfigSpec{
-			EtcdDiscoveryDomain: fmt.Sprintf("%s.tt.testing", name),
-			Platform:            platform,
+			Infra: &apicfgv1.Infrastructure{
+				Status: apicfgv1.InfrastructureStatus{
+					EtcdDiscoveryDomain: fmt.Sprintf("%s.tt.testing", name),
+					PlatformStatus: &apicfgv1.PlatformStatus{
+						Type: platform,
+					},
+					APIServerURL:         fmt.Sprintf("https://api.%s.tt.testing:6443", name),
+					APIServerInternalURL: fmt.Sprintf("https://api-int.%s.tt.testing:6443", name),
+				},
+			},
+			ReleaseImage: "release-reg.io/myuser/myimage:test",
 		},
 	}
 	return cc
@@ -128,15 +133,6 @@ func newContainerRuntimeConfig(name string, ctrconf *mcfgv1.ContainerRuntimeConf
 	}
 }
 
-func newConfigMap(name string) *k8sv1.ConfigMap {
-	return &k8sv1.ConfigMap{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      name,
-			Namespace: "openshift-machine-config-operator",
-		},
-	}
-}
-
 func newImageConfig(name string, regconf *apicfgv1.RegistrySources) *apicfgv1.Image {
 	return &apicfgv1.Image{
 		TypeMeta:   metav1.TypeMeta{APIVersion: apicfgv1.SchemeGroupVersion.String()},
@@ -162,7 +158,7 @@ func newClusterVersionConfig(name, desiredImage string) *apicfgv1.ClusterVersion
 		TypeMeta:   metav1.TypeMeta{APIVersion: apicfgv1.SchemeGroupVersion.String()},
 		ObjectMeta: metav1.ObjectMeta{Name: name, UID: types.UID(utilrand.String(5)), Generation: 1},
 		Status: apicfgv1.ClusterVersionStatus{
-			Desired: apicfgv1.Update{
+			Desired: apicfgv1.Release{
 				Image: desiredImage,
 			},
 		},
@@ -173,24 +169,18 @@ func (f *fixture) newController() *Controller {
 	f.client = fake.NewSimpleClientset(f.objects...)
 	f.imgClient = fakeconfigv1client.NewSimpleClientset(f.imgObjects...)
 	f.operatorClient = fakeoperatorclient.NewSimpleClientset(f.operatorObjects...)
-	f.kubeClient = k8sfake.NewSimpleClientset(f.k8sObjects...)
 
 	i := informers.NewSharedInformerFactory(f.client, noResyncPeriodFunc())
 	ci := configv1informer.NewSharedInformerFactory(f.imgClient, noResyncPeriodFunc())
 	oi := operatorinformer.NewSharedInformerFactory(f.operatorClient, noResyncPeriodFunc())
-	c := New("openshift-machine-config-operator", templateDir,
+	c := New(templateDir,
 		i.Machineconfiguration().V1().MachineConfigPools(),
 		i.Machineconfiguration().V1().ControllerConfigs(),
 		i.Machineconfiguration().V1().ContainerRuntimeConfigs(),
 		ci.Config().V1().Images(),
 		oi.Operator().V1alpha1().ImageContentSourcePolicies(),
 		ci.Config().V1().ClusterVersions(),
-		f.kubeClient, f.client, f.imgClient)
-
-	c.patchContainerRuntimeConfigsFunc = func(name string, patch []byte) error {
-		f.client.Invokes(core.NewRootPatchAction(schema.GroupVersionResource{Version: "v1", Group: "machineconfiguration.openshift.io", Resource: "containerruntimeconfigs"}, name, types.MergePatchType, patch), nil)
-		return nil
-	}
+		k8sfake.NewSimpleClientset(), f.client, f.imgClient)
 
 	c.mcpListerSynced = alwaysReady
 	c.mccrListerSynced = alwaysReady
@@ -242,7 +232,14 @@ func (f *fixture) runExpectError(mcpname string) {
 func (f *fixture) runController(mcpname string, expectError bool) {
 	c := f.newController()
 
-	err := c.syncHandler(mcpname)
+	err := c.syncImgHandler(mcpname)
+	if !expectError && err != nil {
+		f.t.Errorf("error syncing image config: %v", err)
+	} else if expectError && err == nil {
+		f.t.Error("expected error syncing image config, got nil")
+	}
+
+	err = c.syncHandler(mcpname)
 	if !expectError && err != nil {
 		f.t.Errorf("error syncing containerruntimeconfigs: %v", err)
 	} else if expectError && err == nil {
@@ -340,29 +337,34 @@ func (f *fixture) expectUpdateContainerRuntimeConfig(config *mcfgv1.ContainerRun
 	f.actions = append(f.actions, core.NewRootUpdateSubresourceAction(schema.GroupVersionResource{Version: "v1", Group: "machineconfiguration.openshift.io", Resource: "containerruntimeconfigs"}, "status", config))
 }
 
-func (f *fixture) verifyRegistriesConfigAndPolicyJSONContents(t *testing.T, mcName string, imgcfg *apicfgv1.Image, icsp *apioperatorsv1alpha1.ImageContentSourcePolicy, verifyPolicyJSON bool) {
+func (f *fixture) verifyRegistriesConfigAndPolicyJSONContents(t *testing.T, mcName string, imgcfg *apicfgv1.Image, icsp *apioperatorsv1alpha1.ImageContentSourcePolicy, releaseImageReg string, verifyPolicyJSON, verifySearchRegsDropin bool) {
 	icsps := []*apioperatorsv1alpha1.ImageContentSourcePolicy{}
 	if icsp != nil {
 		icsps = append(icsps, icsp)
 	}
 	updatedMC, err := f.client.MachineconfigurationV1().MachineConfigs().Get(context.TODO(), mcName, metav1.GetOptions{})
 	require.NoError(t, err)
-	verifyRegistriesConfigAndPolicyJSONContents(t, updatedMC, mcName, imgcfg, icsps, verifyPolicyJSON)
+	verifyRegistriesConfigAndPolicyJSONContents(t, updatedMC, mcName, imgcfg, icsps, releaseImageReg, verifyPolicyJSON, verifySearchRegsDropin)
 }
 
-func verifyRegistriesConfigAndPolicyJSONContents(t *testing.T, mc *mcfgv1.MachineConfig, mcName string, imgcfg *apicfgv1.Image, icsps []*apioperatorsv1alpha1.ImageContentSourcePolicy, verifyPolicyJSON bool) {
+func verifyRegistriesConfigAndPolicyJSONContents(t *testing.T, mc *mcfgv1.MachineConfig, mcName string, imgcfg *apicfgv1.Image, icsps []*apioperatorsv1alpha1.ImageContentSourcePolicy, releaseImageReg string, verifyPolicyJSON, verifySearchRegsDropin bool) {
 	// This is not testing updateRegistriesConfig, which has its own tests; this verifies the created object contains the expected
 	// configuration file.
+	// First get the valid blocked registries to ensure we don't block the registry where the release image is from
+	blockedRegistries, _ := getValidBlockedRegistries(releaseImageReg, &imgcfg.Spec)
 	expectedRegistriesConf, err := updateRegistriesConfig(templateRegistriesConfig,
 		imgcfg.Spec.RegistrySources.InsecureRegistries,
-		imgcfg.Spec.RegistrySources.BlockedRegistries, icsps)
+		blockedRegistries, icsps)
 	require.NoError(t, err)
 	assert.Equal(t, mcName, mc.ObjectMeta.Name)
 
-	ignCfg, _, err := ign.Parse(mc.Spec.Config.Raw)
+	ignCfg, err := ctrlcommon.ParseAndConvertConfig(mc.Spec.Config.Raw)
 	require.NoError(t, err)
-	if verifyPolicyJSON {
-		// If there is a change to the policy.json file then there will be 2 files
+	if verifyPolicyJSON && verifySearchRegsDropin {
+		// If there is a change to the policy.json AND drop-in search registries file then there will be 3 files
+		require.Len(t, ignCfg.Storage.Files, 3)
+	} else if verifyPolicyJSON || verifySearchRegsDropin {
+		// If there is a change to the policy.json file OR the drop-in search registries file then there will be 2 files
 		require.Len(t, ignCfg.Storage.Files, 2)
 	} else {
 		require.Len(t, ignCfg.Storage.Files, 1)
@@ -372,14 +374,14 @@ func verifyRegistriesConfigAndPolicyJSONContents(t *testing.T, mc *mcfgv1.Machin
 		regfile = ignCfg.Storage.Files[1]
 	}
 	assert.Equal(t, registriesConfigPath, regfile.Node.Path)
-	registriesConf, err := dataurl.DecodeString(regfile.Contents.Source)
+	registriesConf, err := dataurl.DecodeString(*regfile.Contents.Source)
 	require.NoError(t, err)
 	assert.Equal(t, string(expectedRegistriesConf), string(registriesConf.Data))
 
 	// Validate the policy.json contents if a change is expected from the tests
 	if verifyPolicyJSON {
 		expectedPolicyJSON, err := updatePolicyJSON(templatePolicyJSON,
-			imgcfg.Spec.RegistrySources.BlockedRegistries,
+			blockedRegistries,
 			imgcfg.Spec.RegistrySources.AllowedRegistries)
 		require.NoError(t, err)
 		policyfile := ignCfg.Storage.Files[1]
@@ -387,38 +389,53 @@ func verifyRegistriesConfigAndPolicyJSONContents(t *testing.T, mc *mcfgv1.Machin
 			policyfile = ignCfg.Storage.Files[0]
 		}
 		assert.Equal(t, policyConfigPath, policyfile.Node.Path)
-		policyJSON, err := dataurl.DecodeString(policyfile.Contents.Source)
+		policyJSON, err := dataurl.DecodeString(*policyfile.Contents.Source)
 		require.NoError(t, err)
 		assert.Equal(t, string(expectedPolicyJSON), string(policyJSON.Data))
 	}
+
+	// Validate the drop-in search registries file contents if a change is expected from the tests
+	if verifySearchRegsDropin {
+		expectedSearchRegsConf := updateSearchRegistriesConfig(imgcfg.Spec.RegistrySources.ContainerRuntimeSearchRegistries)
+		dropinfile := ignCfg.Storage.Files[2]
+		if dropinfile.Node.Path != searchRegDropInFilePath {
+			dropinfile = ignCfg.Storage.Files[1]
+		}
+		assert.Equal(t, searchRegDropInFilePath, dropinfile.Node.Path)
+		searchRegsConf, err := dataurl.DecodeString(*dropinfile.Contents.Source)
+		require.NoError(t, err)
+		assert.Equal(t, string(expectedSearchRegsConf[0].data), string(searchRegsConf.Data))
+	}
 }
 
-// The pathc bytes to expect when creating/updating a containerruntimeconfig
+// The patch bytes to expect when creating/updating a containerruntimeconfig
 var ctrcfgPatchBytes = []uint8{0x7b, 0x22, 0x6d, 0x65, 0x74, 0x61, 0x64, 0x61, 0x74, 0x61, 0x22, 0x3a, 0x7b, 0x22, 0x66, 0x69, 0x6e, 0x61, 0x6c, 0x69, 0x7a, 0x65, 0x72, 0x73, 0x22, 0x3a, 0x5b, 0x22, 0x39, 0x39, 0x2d, 0x6d, 0x61, 0x73, 0x74, 0x65, 0x72, 0x2d, 0x73, 0x78, 0x32, 0x76, 0x72, 0x2d, 0x63, 0x6f, 0x6e, 0x74, 0x61, 0x69, 0x6e, 0x65, 0x72, 0x72, 0x75, 0x6e, 0x74, 0x69, 0x6d, 0x65, 0x22, 0x5d, 0x7d, 0x7d}
 
 // TestContainerRuntimeConfigCreate ensures that a create happens when an existing containerruntime config is created.
 // It tests that the necessary get, create, and update steps happen in the correct order.
 func TestContainerRuntimeConfigCreate(t *testing.T) {
-	for _, platform := range []string{"aws", "none", "unrecognized"} {
-		t.Run(platform, func(t *testing.T) {
+	for _, platform := range []apicfgv1.PlatformType{apicfgv1.AWSPlatformType, apicfgv1.NonePlatformType, "unrecognized"} {
+		t.Run(string(platform), func(t *testing.T) {
 			f := newFixture(t)
+			f.newController()
 
 			cc := newControllerConfig(ctrlcommon.ControllerConfigName, platform)
 			mcp := helpers.NewMachineConfigPool("master", nil, helpers.MasterSelector, "v0")
-			mcp.ObjectMeta.Labels["custom-crio"] = "my-config"
 			mcp2 := helpers.NewMachineConfigPool("worker", nil, helpers.WorkerSelector, "v0")
-			mcp2.ObjectMeta.Labels["custom-crio"] = "storage-config"
-			ctrcfg1 := newContainerRuntimeConfig("set-log-level", &mcfgv1.ContainerRuntimeConfiguration{LogLevel: "debug", LogSizeMax: resource.MustParse("9k"), OverlaySize: resource.MustParse("3G")}, metav1.AddLabelToSelector(&metav1.LabelSelector{}, "custom-crio", "my-config"))
-			mcs1 := helpers.NewMachineConfig(getManagedKeyCtrCfg(mcp), map[string]string{"node-role": "master"}, "dummy://", []igntypes.File{{}})
-			cm := newConfigMap("crio-capabilities")
+			ctrcfg1 := newContainerRuntimeConfig("set-log-level", &mcfgv1.ContainerRuntimeConfiguration{LogLevel: "debug", LogSizeMax: resource.MustParse("9k"), OverlaySize: resource.MustParse("3G")}, metav1.AddLabelToSelector(&metav1.LabelSelector{}, "pools.operator.machineconfiguration.openshift.io/master", ""))
+			ctrCfgKey, _ := getManagedKeyCtrCfg(mcp, f.client, ctrcfg1)
+			mcs1 := helpers.NewMachineConfig(getManagedKeyCtrCfgDeprecated(mcp), map[string]string{"node-role": "master"}, "dummy://", []ign3types.File{{}})
+			mcs2 := mcs1.DeepCopy()
+			mcs2.Name = ctrCfgKey
 
 			f.ccLister = append(f.ccLister, cc)
 			f.mcpLister = append(f.mcpLister, mcp)
 			f.mcpLister = append(f.mcpLister, mcp2)
 			f.mccrLister = append(f.mccrLister, ctrcfg1)
 			f.objects = append(f.objects, ctrcfg1)
-			f.k8sObjects = append(f.k8sObjects, cm)
 
+			f.expectGetMachineConfigAction(mcs2)
+			f.expectGetMachineConfigAction(mcs1)
 			f.expectGetMachineConfigAction(mcs1)
 			f.expectUpdateContainerRuntimeConfig(ctrcfg1)
 			f.expectCreateMachineConfigAction(mcs1)
@@ -433,26 +450,28 @@ func TestContainerRuntimeConfigCreate(t *testing.T) {
 // TestContainerRuntimeConfigUpdate ensures that an update happens when an existing containerruntime config is updated.
 // It tests that the necessary get, create, and update steps happen in the correct order.
 func TestContainerRuntimeConfigUpdate(t *testing.T) {
-	for _, platform := range []string{"aws", "none", "unrecognized"} {
-		t.Run(platform, func(t *testing.T) {
+	for _, platform := range []apicfgv1.PlatformType{apicfgv1.AWSPlatformType, apicfgv1.NonePlatformType, "unrecognized"} {
+		t.Run(string(platform), func(t *testing.T) {
 			f := newFixture(t)
+			f.newController()
 
 			cc := newControllerConfig(ctrlcommon.ControllerConfigName, platform)
 			mcp := helpers.NewMachineConfigPool("master", nil, helpers.MasterSelector, "v0")
-			mcp.ObjectMeta.Labels["custom-crio"] = "my-config"
 			mcp2 := helpers.NewMachineConfigPool("worker", nil, helpers.WorkerSelector, "v0")
-			mcp2.ObjectMeta.Labels["custom-crio"] = "storage-config"
-			ctrcfg1 := newContainerRuntimeConfig("set-log-level", &mcfgv1.ContainerRuntimeConfiguration{LogLevel: "debug", LogSizeMax: resource.MustParse("9k"), OverlaySize: resource.MustParse("3G")}, metav1.AddLabelToSelector(&metav1.LabelSelector{}, "custom-crio", "my-config"))
-			mcs := helpers.NewMachineConfig(getManagedKeyCtrCfg(mcp), map[string]string{"node-role": "master"}, "dummy://", []igntypes.File{{}})
-			cm := newConfigMap("crio-capabilities")
+			ctrcfg1 := newContainerRuntimeConfig("set-log-level", &mcfgv1.ContainerRuntimeConfiguration{LogLevel: "debug", LogSizeMax: resource.MustParse("9k"), OverlaySize: resource.MustParse("3G")}, metav1.AddLabelToSelector(&metav1.LabelSelector{}, "pools.operator.machineconfiguration.openshift.io/master", ""))
+			keyCtrCfg, _ := getManagedKeyCtrCfg(mcp, f.client, ctrcfg1)
+			mcs := helpers.NewMachineConfig(getManagedKeyCtrCfgDeprecated(mcp), map[string]string{"node-role": "master"}, "dummy://", []ign3types.File{{}})
+			mcsUpdate := mcs.DeepCopy()
+			mcsUpdate.Name = keyCtrCfg
 
 			f.ccLister = append(f.ccLister, cc)
 			f.mcpLister = append(f.mcpLister, mcp)
 			f.mcpLister = append(f.mcpLister, mcp2)
 			f.mccrLister = append(f.mccrLister, ctrcfg1)
 			f.objects = append(f.objects, ctrcfg1)
-			f.k8sObjects = append(f.k8sObjects, cm)
 
+			f.expectGetMachineConfigAction(mcsUpdate)
+			f.expectGetMachineConfigAction(mcs)
 			f.expectGetMachineConfigAction(mcs)
 			f.expectUpdateContainerRuntimeConfig(ctrcfg1)
 			f.expectCreateMachineConfigAction(mcs)
@@ -481,8 +500,7 @@ func TestContainerRuntimeConfigUpdate(t *testing.T) {
 			f.mcpLister = append(f.mcpLister, mcp)
 			f.mcpLister = append(f.mcpLister, mcp2)
 			f.mccrLister = append(f.mccrLister, ctrcfgUpdate)
-			f.objects = append(f.objects, mcs, ctrcfgUpdate)
-			f.k8sObjects = append(f.k8sObjects, cm)
+			f.objects = append(f.objects, mcsUpdate, ctrcfgUpdate)
 
 			c = f.newController()
 			stopCh = make(chan struct{})
@@ -495,9 +513,10 @@ func TestContainerRuntimeConfigUpdate(t *testing.T) {
 				t.Errorf("syncHandler returned: %v", err)
 			}
 
-			f.expectGetMachineConfigAction(mcs)
+			f.expectGetMachineConfigAction(mcsUpdate)
+			f.expectGetMachineConfigAction(mcsUpdate)
 			f.expectUpdateContainerRuntimeConfig(ctrcfgUpdate)
-			f.expectUpdateMachineConfigAction(mcs)
+			f.expectUpdateMachineConfigAction(mcsUpdate)
 			f.expectPatchContainerRuntimeConfig(ctrcfgUpdate, ctrcfgPatchBytes)
 			f.expectUpdateContainerRuntimeConfig(ctrcfgUpdate)
 
@@ -511,17 +530,19 @@ func TestContainerRuntimeConfigUpdate(t *testing.T) {
 // TestImageConfigCreate ensures that a create happens when an image config is created.
 // It tests that the necessary get, create, and update steps happen in the correct order.
 func TestImageConfigCreate(t *testing.T) {
-	for _, platform := range []string{"aws", "none", "unrecognized"} {
-		t.Run(platform, func(t *testing.T) {
+	for _, platform := range []apicfgv1.PlatformType{apicfgv1.AWSPlatformType, apicfgv1.NonePlatformType, "unrecognized"} {
+		t.Run(string(platform), func(t *testing.T) {
 			f := newFixture(t)
 
 			cc := newControllerConfig(ctrlcommon.ControllerConfigName, platform)
 			mcp := helpers.NewMachineConfigPool("master", nil, helpers.MasterSelector, "v0")
 			mcp2 := helpers.NewMachineConfigPool("worker", nil, helpers.WorkerSelector, "v0")
-			imgcfg1 := newImageConfig("cluster", &apicfgv1.RegistrySources{InsecureRegistries: []string{"blah.io"}, AllowedRegistries: []string{"allow.io"}})
+			imgcfg1 := newImageConfig("cluster", &apicfgv1.RegistrySources{InsecureRegistries: []string{"blah.io"}, AllowedRegistries: []string{"allow.io"}, ContainerRuntimeSearchRegistries: []string{"search-reg.io"}})
 			cvcfg1 := newClusterVersionConfig("version", "test.io/myuser/myimage:test")
-			mcs1 := helpers.NewMachineConfig(getManagedKeyReg(mcp), map[string]string{"node-role": "master"}, "dummy://", []igntypes.File{{}})
-			mcs2 := helpers.NewMachineConfig(getManagedKeyReg(mcp2), map[string]string{"node-role": "worker"}, "dummy://", []igntypes.File{{}})
+			keyReg1, _ := getManagedKeyReg(mcp, nil)
+			keyReg2, _ := getManagedKeyReg(mcp2, nil)
+			mcs1 := helpers.NewMachineConfig(keyReg1, map[string]string{"node-role": "master"}, "dummy://", []ign3types.File{{}})
+			mcs2 := helpers.NewMachineConfig(keyReg2, map[string]string{"node-role": "worker"}, "dummy://", []ign3types.File{{}})
 
 			f.ccLister = append(f.ccLister, cc)
 			f.mcpLister = append(f.mcpLister, mcp)
@@ -530,24 +551,19 @@ func TestImageConfigCreate(t *testing.T) {
 			f.cvLister = append(f.cvLister, cvcfg1)
 			f.imgObjects = append(f.imgObjects, imgcfg1)
 
+			f.expectGetMachineConfigAction(mcs1)
+			f.expectGetMachineConfigAction(mcs1)
 			f.expectGetMachineConfigAction(mcs1)
 			f.expectCreateMachineConfigAction(mcs1)
 			f.expectGetMachineConfigAction(mcs2)
+			f.expectGetMachineConfigAction(mcs2)
+			f.expectGetMachineConfigAction(mcs2)
 			f.expectCreateMachineConfigAction(mcs2)
 
-			c := f.newController()
-			stopCh := make(chan struct{})
-
-			err := c.syncImgHandler("cluster")
-			if err != nil {
-				t.Errorf("syncImgHandler returned %v", err)
-			}
-
-			f.validateActions()
-			close(stopCh)
+			f.run("cluster")
 
 			for _, mcName := range []string{mcs1.Name, mcs2.Name} {
-				f.verifyRegistriesConfigAndPolicyJSONContents(t, mcName, imgcfg1, nil, true)
+				f.verifyRegistriesConfigAndPolicyJSONContents(t, mcName, imgcfg1, nil, cc.Spec.ReleaseImage, true, true)
 			}
 		})
 	}
@@ -556,17 +572,23 @@ func TestImageConfigCreate(t *testing.T) {
 // TestImageConfigUpdate ensures that an update happens when an existing image config is updated.
 // It tests that the necessary get, create, and update steps happen in the correct order.
 func TestImageConfigUpdate(t *testing.T) {
-	for _, platform := range []string{"aws", "none", "unrecognized"} {
-		t.Run(platform, func(t *testing.T) {
+	for _, platform := range []apicfgv1.PlatformType{apicfgv1.AWSPlatformType, apicfgv1.NonePlatformType, "unrecognized"} {
+		t.Run(string(platform), func(t *testing.T) {
 			f := newFixture(t)
 
 			cc := newControllerConfig(ctrlcommon.ControllerConfigName, platform)
 			mcp := helpers.NewMachineConfigPool("master", nil, helpers.MasterSelector, "v0")
 			mcp2 := helpers.NewMachineConfigPool("worker", nil, helpers.WorkerSelector, "v0")
-			imgcfg1 := newImageConfig("cluster", &apicfgv1.RegistrySources{InsecureRegistries: []string{"blah.io"}, AllowedRegistries: []string{"allow.io"}})
+			imgcfg1 := newImageConfig("cluster", &apicfgv1.RegistrySources{InsecureRegistries: []string{"blah.io"}, AllowedRegistries: []string{"allow.io"}, ContainerRuntimeSearchRegistries: []string{"search-reg.io"}})
 			cvcfg1 := newClusterVersionConfig("version", "test.io/myuser/myimage:test")
-			mcs1 := helpers.NewMachineConfig(getManagedKeyReg(mcp), map[string]string{"node-role": "master"}, "dummy://", []igntypes.File{{}})
-			mcs2 := helpers.NewMachineConfig(getManagedKeyReg(mcp2), map[string]string{"node-role": "worker"}, "dummy://", []igntypes.File{{}})
+			keyReg1, _ := getManagedKeyReg(mcp, nil)
+			keyReg2, _ := getManagedKeyReg(mcp2, nil)
+			mcs1 := helpers.NewMachineConfig(getManagedKeyRegDeprecated(mcp), map[string]string{"node-role": "master"}, "dummy://", []ign3types.File{{}})
+			mcs2 := helpers.NewMachineConfig(getManagedKeyRegDeprecated(mcp2), map[string]string{"node-role": "worker"}, "dummy://", []ign3types.File{{}})
+			mcs1Update := mcs1.DeepCopy()
+			mcs2Update := mcs2.DeepCopy()
+			mcs1Update.Name = keyReg1
+			mcs2Update.Name = keyReg2
 
 			f.ccLister = append(f.ccLister, cc)
 			f.mcpLister = append(f.mcpLister, mcp)
@@ -575,8 +597,12 @@ func TestImageConfigUpdate(t *testing.T) {
 			f.cvLister = append(f.cvLister, cvcfg1)
 			f.imgObjects = append(f.imgObjects, imgcfg1)
 
+			f.expectGetMachineConfigAction(mcs1Update)
+			f.expectGetMachineConfigAction(mcs1)
 			f.expectGetMachineConfigAction(mcs1)
 			f.expectCreateMachineConfigAction(mcs1)
+			f.expectGetMachineConfigAction(mcs2Update)
+			f.expectGetMachineConfigAction(mcs2)
 			f.expectGetMachineConfigAction(mcs2)
 			f.expectCreateMachineConfigAction(mcs2)
 
@@ -591,8 +617,8 @@ func TestImageConfigUpdate(t *testing.T) {
 			f.validateActions()
 			close(stopCh)
 
-			for _, mcName := range []string{mcs1.Name, mcs2.Name} {
-				f.verifyRegistriesConfigAndPolicyJSONContents(t, mcName, imgcfg1, nil, true)
+			for _, mcName := range []string{mcs1Update.Name, mcs2Update.Name} {
+				f.verifyRegistriesConfigAndPolicyJSONContents(t, mcName, imgcfg1, nil, cc.Spec.ReleaseImage, true, true)
 			}
 
 			// Perform Update
@@ -608,7 +634,7 @@ func TestImageConfigUpdate(t *testing.T) {
 			f.imgLister = append(f.imgLister, imgcfgUpdate)
 			f.cvLister = append(f.cvLister, cvcfg1)
 			f.imgObjects = append(f.imgObjects, imgcfgUpdate)
-			f.objects = append(f.objects, mcs1, mcs2)
+			f.objects = append(f.objects, mcs1Update, mcs2Update)
 
 			c = f.newController()
 			stopCh = make(chan struct{})
@@ -621,17 +647,19 @@ func TestImageConfigUpdate(t *testing.T) {
 				t.Errorf("syncImgHandler returned: %v", err)
 			}
 
-			f.expectGetMachineConfigAction(mcs1)
-			f.expectUpdateMachineConfigAction(mcs1)
-			f.expectGetMachineConfigAction(mcs2)
-			f.expectUpdateMachineConfigAction(mcs2)
+			f.expectGetMachineConfigAction(mcs1Update)
+			f.expectGetMachineConfigAction(mcs1Update)
+			f.expectUpdateMachineConfigAction(mcs1Update)
+			f.expectGetMachineConfigAction(mcs2Update)
+			f.expectGetMachineConfigAction(mcs2Update)
+			f.expectUpdateMachineConfigAction(mcs2Update)
 
 			f.validateActions()
 
 			close(stopCh)
 
-			for _, mcName := range []string{mcs1.Name, mcs2.Name} {
-				f.verifyRegistriesConfigAndPolicyJSONContents(t, mcName, imgcfgUpdate, nil, true)
+			for _, mcName := range []string{mcs1Update.Name, mcs2Update.Name} {
+				f.verifyRegistriesConfigAndPolicyJSONContents(t, mcName, imgcfgUpdate, nil, cc.Spec.ReleaseImage, true, true)
 			}
 		})
 	}
@@ -640,8 +668,8 @@ func TestImageConfigUpdate(t *testing.T) {
 // TestICSPUpdate ensures that an update happens when an existing ICSP is updated.
 // It tests that the necessary get, create, and update steps happen in the correct order.
 func TestICSPUpdate(t *testing.T) {
-	for _, platform := range []string{"aws", "none", "unrecognized"} {
-		t.Run(platform, func(t *testing.T) {
+	for _, platform := range []apicfgv1.PlatformType{apicfgv1.AWSPlatformType, apicfgv1.NonePlatformType, "unrecognized"} {
+		t.Run(string(platform), func(t *testing.T) {
 			f := newFixture(t)
 
 			cc := newControllerConfig(ctrlcommon.ControllerConfigName, platform)
@@ -649,11 +677,17 @@ func TestICSPUpdate(t *testing.T) {
 			mcp2 := helpers.NewMachineConfigPool("worker", nil, helpers.WorkerSelector, "v0")
 			imgcfg1 := newImageConfig("cluster", &apicfgv1.RegistrySources{InsecureRegistries: []string{"blah.io"}})
 			cvcfg1 := newClusterVersionConfig("version", "test.io/myuser/myimage:test")
-			mcs1 := helpers.NewMachineConfig(getManagedKeyReg(mcp), map[string]string{"node-role": "master"}, "dummy://", []igntypes.File{{}})
-			mcs2 := helpers.NewMachineConfig(getManagedKeyReg(mcp2), map[string]string{"node-role": "worker"}, "dummy://", []igntypes.File{{}})
+			keyReg1, _ := getManagedKeyReg(mcp, nil)
+			keyReg2, _ := getManagedKeyReg(mcp2, nil)
+			mcs1 := helpers.NewMachineConfig(getManagedKeyRegDeprecated(mcp), map[string]string{"node-role": "master"}, "dummy://", []ign3types.File{{}})
+			mcs2 := helpers.NewMachineConfig(getManagedKeyRegDeprecated(mcp2), map[string]string{"node-role": "worker"}, "dummy://", []ign3types.File{{}})
 			icsp := newICSP("built-in", []apioperatorsv1alpha1.RepositoryDigestMirrors{
 				{Source: "built-in-source.example.com", Mirrors: []string{"built-in-mirror.example.com"}},
 			})
+			mcs1Update := mcs1.DeepCopy()
+			mcs2Update := mcs2.DeepCopy()
+			mcs1Update.Name = keyReg1
+			mcs2Update.Name = keyReg2
 
 			f.ccLister = append(f.ccLister, cc)
 			f.mcpLister = append(f.mcpLister, mcp)
@@ -664,8 +698,12 @@ func TestICSPUpdate(t *testing.T) {
 			f.imgObjects = append(f.imgObjects, imgcfg1)
 			f.operatorObjects = append(f.operatorObjects, icsp)
 
+			f.expectGetMachineConfigAction(mcs1Update)
+			f.expectGetMachineConfigAction(mcs1)
 			f.expectGetMachineConfigAction(mcs1)
 			f.expectCreateMachineConfigAction(mcs1)
+			f.expectGetMachineConfigAction(mcs2Update)
+			f.expectGetMachineConfigAction(mcs2)
 			f.expectGetMachineConfigAction(mcs2)
 			f.expectCreateMachineConfigAction(mcs2)
 
@@ -680,8 +718,8 @@ func TestICSPUpdate(t *testing.T) {
 			f.validateActions()
 			close(stopCh)
 
-			for _, mcName := range []string{mcs1.Name, mcs2.Name} {
-				f.verifyRegistriesConfigAndPolicyJSONContents(t, mcName, imgcfg1, icsp, false)
+			for _, mcName := range []string{mcs1Update.Name, mcs2Update.Name} {
+				f.verifyRegistriesConfigAndPolicyJSONContents(t, mcName, imgcfg1, icsp, cc.Spec.ReleaseImage, false, false)
 			}
 
 			// Perform Update
@@ -699,7 +737,7 @@ func TestICSPUpdate(t *testing.T) {
 			f.imgLister = append(f.imgLister, imgcfg1)
 			f.icspLister = append(f.icspLister, icspUpdate)
 			f.cvLister = append(f.cvLister, cvcfg1)
-			f.objects = append(f.objects, mcs1, mcs2)
+			f.objects = append(f.objects, mcs1Update, mcs2Update)
 			f.imgObjects = append(f.imgObjects, imgcfg1)
 			f.operatorObjects = append(f.operatorObjects, icspUpdate)
 
@@ -714,27 +752,27 @@ func TestICSPUpdate(t *testing.T) {
 				t.Errorf("syncImgHandler returned: %v", err)
 			}
 
-			f.expectGetMachineConfigAction(mcs1)
-			f.expectUpdateMachineConfigAction(mcs1)
-			f.expectGetMachineConfigAction(mcs2)
-			f.expectUpdateMachineConfigAction(mcs2)
+			f.expectGetMachineConfigAction(mcs1Update)
+			f.expectGetMachineConfigAction(mcs1Update)
+			f.expectUpdateMachineConfigAction(mcs1Update)
+			f.expectGetMachineConfigAction(mcs2Update)
+			f.expectGetMachineConfigAction(mcs2Update)
+			f.expectUpdateMachineConfigAction(mcs2Update)
 
 			f.validateActions()
 
 			close(stopCh)
 
-			for _, mcName := range []string{mcs1.Name, mcs2.Name} {
-				f.verifyRegistriesConfigAndPolicyJSONContents(t, mcName, imgcfg1, icspUpdate, false)
+			for _, mcName := range []string{mcs1Update.Name, mcs2Update.Name} {
+				f.verifyRegistriesConfigAndPolicyJSONContents(t, mcName, imgcfg1, icspUpdate, cc.Spec.ReleaseImage, false, false)
 			}
 		})
 	}
 }
 
 func TestRunImageBootstrap(t *testing.T) {
-	emptyImgCfg := &apicfgv1.Image{}
-
-	for _, platform := range []string{"aws", "none", "unrecognized"} {
-		t.Run(platform, func(t *testing.T) {
+	for _, platform := range []apicfgv1.PlatformType{apicfgv1.AWSPlatformType, apicfgv1.NonePlatformType, "unrecognized"} {
+		t.Run(string(platform), func(t *testing.T) {
 			cc := newControllerConfig(ctrlcommon.ControllerConfigName, platform)
 			pools := []*mcfgv1.MachineConfigPool{
 				helpers.NewMachineConfigPool("master", nil, helpers.MasterSelector, "v0"),
@@ -746,12 +784,17 @@ func TestRunImageBootstrap(t *testing.T) {
 					{Source: "built-in-source.example.com", Mirrors: []string{"local-mirror.local"}},
 				}),
 			}
+			// Adding the release-image registry "release-reg.io" to the list of blocked registries to ensure that is it not added to
+			// both registries.conf and policy.json as blocked
+			imgCfg := newImageConfig("cluster", &apicfgv1.RegistrySources{InsecureRegistries: []string{"insecure-reg-1.io", "insecure-reg-2.io"}, BlockedRegistries: []string{"blocked-reg.io", "release-reg.io"}, ContainerRuntimeSearchRegistries: []string{"search-reg.io"}})
 
-			mcs, err := RunImageBootstrap("../../../templates", cc, pools, icspRules)
+			mcs, err := RunImageBootstrap("../../../templates", cc, pools, icspRules, imgCfg)
 			require.NoError(t, err)
 			require.Len(t, mcs, len(pools))
+
 			for i := range pools {
-				verifyRegistriesConfigAndPolicyJSONContents(t, mcs[i], getManagedKeyReg(pools[i]), emptyImgCfg, icspRules, false)
+				keyReg, _ := getManagedKeyReg(pools[i], nil)
+				verifyRegistriesConfigAndPolicyJSONContents(t, mcs[i], keyReg, imgCfg, icspRules, cc.Spec.ReleaseImage, true, true)
 			}
 		})
 	}
@@ -790,7 +833,7 @@ func TestRegistriesValidation(t *testing.T) {
 	for _, test := range failureTests {
 		imgcfg := newImageConfig(test.name, test.config)
 		cvcfg := newClusterVersionConfig("version", "blah.io/myuser/myimage:test")
-		blocked, err := getValidBlockedRegistries(&cvcfg.Status, &imgcfg.Spec)
+		blocked, err := getValidBlockedRegistries(cvcfg.Status.Desired.Image, &imgcfg.Spec)
 		if err == nil {
 			t.Errorf("%s: failed", test.name)
 		}
@@ -805,7 +848,7 @@ func TestRegistriesValidation(t *testing.T) {
 	for _, test := range successTests {
 		imgcfg := newImageConfig(test.name, test.config)
 		cvcfg := newClusterVersionConfig("version", "blah.io/myuser/myimage:test")
-		blocked, err := getValidBlockedRegistries(&cvcfg.Status, &imgcfg.Spec)
+		blocked, err := getValidBlockedRegistries(cvcfg.Status.Desired.Image, &imgcfg.Spec)
 		if err != nil {
 			t.Errorf("%s: failed", test.name)
 		}
