@@ -188,7 +188,7 @@ func getManagedKeyCtrCfg(pool *mcfgv1.MachineConfigPool, client mcfgclientset.In
 		return "", fmt.Errorf("error listing container runtime configs: %v", err)
 	}
 	// If there is no ctrcfg in the list, return the default MC name with no suffix
-	if ctrcfgList == nil || ctrcfgList.Items == nil {
+	if ctrcfgList == nil || len(ctrcfgList.Items) == 0 {
 		return ctrlcommon.GetManagedKey(pool, client, "99", "containerruntime", getManagedKeyCtrCfgDeprecated(pool))
 	}
 	for _, ctrcfg := range ctrcfgList.Items {
@@ -208,15 +208,18 @@ func getManagedKeyCtrCfg(pool *mcfgv1.MachineConfigPool, client mcfgclientset.In
 
 	// If we are here, this means that a new ctrcfg was created, so we have to calculate the suffix value for its MC name
 	suffixNum := 0
-	// Get the suffix value of the second to last item in the ctrcfg list. We use the second to last item because
-	// the most recent ctrcfg being created has already been added to the list and is the ctrcfg object that we are
-	// trying to figure out the suffix value for
-	val, ok := ctrcfgList.Items[len(ctrcfgList.Items)-2].GetAnnotations()[ctrlcommon.MCNameSuffixAnnotationKey]
-	if ok {
-		// Convert the suffix value to int so we can add 1 to it for the MC name for the latest ctrcfg object
-		suffixNum, err = strconv.Atoi(val)
-		if err != nil {
-			return "", fmt.Errorf("error converting %s to int: %v", val, err)
+	// Go through the list of ctrcfg objects created and get the max suffix value currently created
+	for _, item := range ctrcfgList.Items {
+		val, ok := item.GetAnnotations()[ctrlcommon.MCNameSuffixAnnotationKey]
+		if ok {
+			// Convert the suffix value to int so we can look through the list and grab the max suffix created so far
+			intVal, err := strconv.Atoi(val)
+			if err != nil {
+				return "", fmt.Errorf("error converting %s to int: %v", val, err)
+			}
+			if intVal > suffixNum {
+				suffixNum = intVal
+			}
 		}
 	}
 	// The max suffix value that we can go till with this logic is 9 - this means that a user can create up to 10 different ctrcfg CRs.
